import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * A compressor implementing BurrowsWheeler compression
 */

public class BurrowsWheelerCompressor  {
	
	public void compress(BitReader reader, BitWriter writer) throws IOException 
	{
		int inputLength = reader.length();
		byte[] input = new byte[inputLength];
		reader.readBytes(input, 0, inputLength);
		
		BurrowsWheelerTransformer bwt = new BurrowsWheelerTransformer();

		byte[] bwtTransformedResult;
		bwtTransformedResult = bwt.transform(input);

		MoveToFrontTransformer mtf = new MoveToFrontTransformer();
		byte[] mtfTransformedResult;
		mtfTransformedResult = mtf.transform(bwtTransformedResult);

		HuffmanEncode he = new HuffmanEncode();
		BitReader readerMTF = new BitReader(new ByteArrayInputStream(mtfTransformedResult));
		
		he.encode(readerMTF, writer);
	}

	public void expand(BitReader reader, BitWriter writer) throws IOException 
	{
		if (reader.length() == 0)
			return;
		
		HuffmanDecode hd = new HuffmanDecode();
		hd.HuffmanTree(reader);
		int fileBytes = reader.readInt();
		
		byte[] expandedFromHC = new byte[fileBytes];
		for (int i = 0; i < fileBytes; i++)
			expandedFromHC[i] = hd.decode(reader);

		MoveToFrontTransformer mtf = new MoveToFrontTransformer();
		byte[] invertedFromMTF = mtf.invertTransform(expandedFromHC);

		BurrowsWheelerTransformer bwt = new BurrowsWheelerTransformer();
		byte[] invertedFromBW = bwt.invertTransform(invertedFromMTF);
		
		for (int i = 0; i < invertedFromBW.length; i++)
			writer.writeByte(invertedFromBW[i]);
		
		writer.flush();
	}
}
